
#ifndef _WB_MODEL_VIEW_FORM_H_
#define _WB_MODEL_VIEW_FORM_H_

#include "wb_context.h"



namespace wb {
  class Floater;
  class Tooltip;

  class MYSQLWBBACKEND_PUBLIC_FUNC ModelViewForm : public bec::UIForm
  {
  public:
    ModelViewForm(WBComponent *owner, const model_ViewRef &view);
    virtual ~ModelViewForm();

    void attach_canvas_view(mdc::CanvasView *cview);

    virtual bool is_main_form() { return true; }

    mdc::CanvasView *get_view() { return _view; }
    model_ViewRef &get_model_view() { return _model_view; }

    grt::DictRef get_model_options() { return _model_view->owner()->options(); }
    grt::DictRef get_view_options() { return _model_view->options(); }

    virtual grt::UndoManager *get_undo_manager();

    bec::Clipboard *get_clipboard();

    virtual std::string get_title() { return _model_view->name().c_str(); }
    
    virtual bool can_copy();
    virtual bool can_paste();
    virtual bool can_delete();

    virtual void cut();
    virtual void copy();
    virtual void paste();
    virtual void delete_selection();
    int delete_selection(bool noquestions);

    virtual std::string get_edit_target_name();

    grt::ListRef<model_Object> get_selection();
    grt::ListRef<model_Object> get_copiable_selection();
    bool has_selection();

    double get_zoom();
    void set_zoom(double zoom);

    void set_button_callback(const sigc::slot<bool, ModelViewForm*, mdc::MouseButton, bool, mdc::Point, mdc::EventState> &cb);
    void set_motion_callback(const sigc::slot<bool, ModelViewForm*, mdc::Point, mdc::EventState> &cb);
    void set_reset_tool_callback(const sigc::slot<void, ModelViewForm*> &cb);

    std::string get_tool() { return _tool; }
    void set_tool(const std::string &tool);
    void reset_tool(bool notify);
    void set_tool_argument(const std::string &option, const std::string &value);
    std::string get_tool_argument(const std::string &option);

    void activate_figure(const model_ObjectRef &object);
    void focus_and_make_visible(const model_ObjectRef &object, bool select);

    void set_cursor(const std::string &cursor);
    inline const std::string &get_cursor() { return _cursor; }

    // events
    void handle_mouse_move(int x, int y, mdc::EventState state);
    void handle_mouse_button(mdc::MouseButton button, bool press, int x, int y, mdc::EventState state);
    void handle_key(const mdc::KeyInfo &key, bool press, mdc::EventState state);


    // drag&drop
    bool accepts_drop(int x, int y, const std::string &type, const std::list<GrtObjectRef> &objects);
    bool accepts_drop(int x, int y, const std::string &type, const std::string &text);

    bool perform_drop(int x, int y, const std::string &type, const std::list<GrtObjectRef> &objects);
    bool perform_drop(int x, int y, const std::string &type, const std::string &text);


    model_LayerRef get_layer_at(const mdc::Point &pos, mdc::Point &offset);
    model_LayerRef get_layer_bounding(const mdc::Rect &rect, mdc::Point &offset);
    model_ObjectRef get_object_at(const mdc::Point &pos);


    mdc::Layer *get_floater_layer();
    void add_floater(Floater *floater);


    void enable_panning(bool flag);

    sigc::signal<void, std::string> signal_tool_argument_changed() { return _tool_argument_changed; }

    WBComponent *get_owner() { return _owner; }

  protected:
    mdc::CanvasView *_view;
    mdc::Layer *_main_layer;
    mdc::Layer *_floater_layer;
    WBComponent *_owner;
    model_ViewRef _model_view;
    std::string _tool;
    std::string _cursor;
    std::map<std::string, std::string> _tool_args;
    std::vector<WBShortcut> _shortcuts;

    std::map<grt::internal::Value*, mdc::Point> _old_positions;
    grt::UndoAction *_latest_drag_action;

    double _paste_offset;

    Tooltip *_tooltip;
    mdc::CanvasItem *_last_over_item;
    bec::GRTManager::Timer *_tooltip_timer;

    sigc::signal<void, std::string> _tool_argument_changed;

    sigc::slot<bool, ModelViewForm*, mdc::MouseButton, bool, mdc::Point, mdc::EventState> _handle_button;
    sigc::slot<bool, ModelViewForm*, mdc::Point, mdc::EventState> _handle_motion;
    sigc::slot<void, ModelViewForm*> _reset_tool;


    bool _drag_panning;
    bool _space_panning;

    // saved state for tmp panning
    std::string _old_tool;
    std::string _old_cursor;
    sigc::slot<void, ModelViewForm*> _old_reset_tool;
    sigc::slot<bool, ModelViewForm*, mdc::MouseButton, bool, mdc::Point, mdc::EventState> _old_handle_button;
    sigc::slot<bool, ModelViewForm*, mdc::Point, mdc::EventState> _old_handle_motion;

    void clipboard_changed();

    bool relocate_selection();

    bool show_tooltip(const mdc::Point &pos);

    void begin_selection_drag();
    void end_selection_drag();
    
    
    //void hover_in_figure(mdc::CanvasItem *item, mdc::Point pos);
    //void hover_out_figure(mdc::CanvasItem *item);
  };

};


#endif
