#include "stdafx.h"
#include "mysql_relationship_editor.h"


using namespace bec;


RelationshipEditorBE::RelationshipEditorBE(GRTManager *grtm, const workbench_physical_ConnectionRef &relationship)
: BaseEditor(grtm, relationship), _relationship(relationship)
{
}


void RelationshipEditorBE::set_caption(const std::string &caption)
{
  if (add_undo_object_text_change(_relationship, "caption"))
    get_undo_manager()->set_action_description(_("Change Relationship Caption"));
  _relationship->caption(caption);
}


std::string RelationshipEditorBE::get_caption()
{
  return _relationship->caption();
}


void RelationshipEditorBE::set_extra_caption(const std::string &caption)
{
  if (add_undo_object_text_change(_relationship, "extraCaption"))
    get_undo_manager()->set_action_description(_("Change Relationship 2nd Caption"));
  _relationship->extraCaption(caption);
}


std::string RelationshipEditorBE::get_extra_caption()
{
  return _relationship->extraCaption();
}


void RelationshipEditorBE::set_mandatory(bool flag)
{
  add_undo(new grt::UndoObjectChangeAction(_relationship->foreignKey(), "mandatory"), _("Change Mandatory"));
  _relationship->foreignKey()->mandatory(flag?1:0);
}


bool RelationshipEditorBE::get_mandatory()
{
  return _relationship->foreignKey()->mandatory() != 0;
}


void RelationshipEditorBE::set_ref_mandatory(bool flag)
{
  add_undo(new grt::UndoObjectChangeAction(_relationship->foreignKey(), "referencedMandatory"), _("Change Referred Mandatory"));
  _relationship->foreignKey()->referencedMandatory(flag?1:0);
}


bool RelationshipEditorBE::get_ref_mandatory()
{
  return _relationship->foreignKey()->referencedMandatory() != 0;
}


void RelationshipEditorBE::set_to_many(bool flag)
{
  add_undo(new grt::UndoObjectChangeAction(_relationship->foreignKey(), "many"), _("Change Relationship Cardinality"));
  _relationship->foreignKey()->many(flag?1:0);
}


bool RelationshipEditorBE::get_to_many()
{
  return _relationship->foreignKey()->many() != 0;
}


void RelationshipEditorBE::set_comment(const std::string &comment)
{
  if (add_undo_object_text_change(_relationship, "comment"))
    get_undo_manager()->set_action_description(_("Change Relationship Comment"));
  _relationship->comment(comment);
}


std::string RelationshipEditorBE::get_comment()
{
  return _relationship->comment();
}


bool RelationshipEditorBE::get_visible()
{
  return _relationship->visible() != 0;
}

bool RelationshipEditorBE::get_splitted()
{
  return _relationship->drawSplit() != 0;
}

void RelationshipEditorBE::set_visible(bool flag)
{
  add_undo(new grt::UndoObjectChangeAction(_relationship, "visible"), _("Change Relationship Visibility"));
  _relationship->visible(flag?1:0);
}


void RelationshipEditorBE::set_splitted(bool flag)
{
  add_undo(new grt::UndoObjectChangeAction(_relationship, "drawSplit"), _("Change Relationship Visibility"));
  _relationship->drawSplit(flag?1:0);
}
